@@ -1,45 +1,45 @@
-# -*- coding: utf-8 -*-
 """
 Functions for dealing with markup text
 """
 
-import warnings
 import re
-import six
-from six import moves
+from html.entities import name2codepoint
+from typing import Iterable, Match, AnyStr, Optional, Pattern, Tuple, Union
+from urllib.parse import urljoin
 
-from w3lib.util import to_bytes, to_unicode
+from w3lib.util import to_unicode
 from w3lib.url import safe_url_string
+from w3lib._types import StrOrBytes
 
-_ent_re = re.compile(r'&((?P<named>[a-z\d]+)|#(?P<dec>\d+)|#x(?P<hex>[a-f\d]+))(?P<semicolon>;?)', re.IGNORECASE)
-_tag_re = re.compile(r'<[a-zA-Z\/!].*?>', re.DOTALL)
-_baseurl_re = re.compile(six.u(r'<base\s[^>]*href\s*=\s*[\"\']\s*([^\"\'\s]+)\s*[\"\']'), re.I)
-_meta_refresh_re = re.compile(six.u(r'<meta\s[^>]*http-equiv[^>]*refresh[^>]*content\s*=\s*(?P<quote>["\'])(?P<int>(\d*\.)?\d+)\s*;\s*url=\s*(?P<url>.*?)(?P=quote)'), re.DOTALL | re.IGNORECASE)
-_cdata_re = re.compile(r'((?P<cdata_s><!\[CDATA\[)(?P<cdata_d>.*?)(?P<cdata_e>\]\]>))', re.DOTALL)
+_ent_re = re.compile(
+    r"&((?P<named>[a-z\d]+)|#(?P<dec>\d+)|#x(?P<hex>[a-f\d]+))(?P<semicolon>;?)",
+    re.IGNORECASE,
+)
+_tag_re = re.compile(r"<[a-zA-Z\/!].*?>", re.DOTALL)
+_baseurl_re = re.compile(r"<base\s[^>]*href\s*=\s*[\"\']\s*([^\"\'\s]+)\s*[\"\']", re.I)
+_meta_refresh_re = re.compile(
+    r'<meta\s[^>]*http-equiv[^>]*refresh[^>]*content\s*=\s*(?P<quote>["\'])(?P<int>(\d*\.)?\d+)\s*;\s*url=\s*(?P<url>.*?)(?P=quote)',
+    re.DOTALL | re.IGNORECASE,
+)
+_meta_refresh_re2 = re.compile(
+    r'<meta\s[^>]*content\s*=\s*(?P<quote>["\'])(?P<int>(\d*\.)?\d+)\s*;\s*url=\s*(?P<url>.*?)(?P=quote)[^>]*?\shttp-equiv\s*=[^>]*refresh',
+    re.DOTALL | re.IGNORECASE,
+)
 
-HTML5_WHITESPACE = ' \t\n\r\x0c'
+_cdata_re = re.compile(
+    r"((?P<cdata_s><!\[CDATA\[)(?P<cdata_d>.*?)(?P<cdata_e>\]\]>))", re.DOTALL
+)
+
+HTML5_WHITESPACE = " \t\n\r\x0c"
 
 
-def remove_entities(text, keep=(), remove_illegal=True, encoding='utf-8'):
-    r"""
-
-    .. warning::
-
-        This function is deprecated and will be removed in future.
-        Please use :func:`replace_entities` instead.
-    """
-
-    warnings.warn(
-        "`w3lib.html.remove_entities` function is deprecated and "
-        "will be removed in future releases. Please use "
-        "`w3lib.html.replace_entities` instead.",
-        DeprecationWarning
-    )
-
-    return replace_entities(text, keep, remove_illegal, encoding)
-
-def replace_entities(text, keep=(), remove_illegal=True, encoding='utf-8'):
-    u"""Remove entities from the given `text` by converting them to their
+def replace_entities(
+    text: AnyStr,
+    keep: Iterable[str] = (),
+    remove_illegal: bool = True,
+    encoding: str = "utf-8",
+) -> str:
+    """Remove entities from the given `text` by converting them to their
     corresponding unicode character.
 
     `text` can be a unicode string or a byte string encoded in the given
@@ -59,47 +59,51 @@ def replace_entities(text, keep=(), remove_illegal=True, encoding='utf-8'):
 
     >>> import w3lib.html
     >>> w3lib.html.replace_entities(b'Price: &pound;100')
-    u'Price: \\xa3100'
+    'Price: \\xa3100'
     >>> print(w3lib.html.replace_entities(b'Price: &pound;100'))
     Price: £100
     >>>
 
     """
 
-    def convert_entity(m):
+    def convert_entity(m: Match) -> str:
         groups = m.groupdict()
-        if groups.get('dec'):
-            number = int(groups['dec'], 10)
-        elif groups.get('hex'):
-            number = int(groups['hex'], 16)
-        elif groups.get('named'):
-            entity_name = groups['named']
+        number = None
+        if groups.get("dec"):
+            number = int(groups["dec"], 10)
+        elif groups.get("hex"):
+            number = int(groups["hex"], 16)
+        elif groups.get("named"):
+            entity_name = groups["named"]
             if entity_name.lower() in keep:
                 return m.group(0)
             else:
-                number = (moves.html_entities.name2codepoint.get(entity_name) or
-                    moves.html_entities.name2codepoint.get(entity_name.lower()))
+                number = name2codepoint.get(entity_name) or name2codepoint.get(
+                    entity_name.lower()
+                )
         if number is not None:
             # Numeric character references in the 80-9F range are typically
             # interpreted by browsers as representing the characters mapped
             # to bytes 80-9F in the Windows-1252 encoding. For more info
             # see: http://en.wikipedia.org/wiki/Character_encodings_in_HTML
             try:
-                if 0x80 <= number <= 0x9f:
-                    return six.int2byte(number).decode('cp1252')
+                if 0x80 <= number <= 0x9F:
+                    return bytes((number,)).decode("cp1252")
                 else:
-                    return six.unichr(number)
+                    return chr(number)
             except ValueError:
                 pass
 
-        return u'' if remove_illegal and groups.get('semicolon') else m.group(0)
+        return "" if remove_illegal and groups.get("semicolon") else m.group(0)
 
     return _ent_re.sub(convert_entity, to_unicode(text, encoding))
 
-def has_entities(text, encoding=None):
+
+def has_entities(text: AnyStr, encoding: Optional[str] = None) -> bool:
     return bool(_ent_re.search(to_unicode(text, encoding)))
 
-def replace_tags(text, token='', encoding=None):
+
+def replace_tags(text: AnyStr, token: str = "", encoding: Optional[str] = None) -> str:
     """Replace all markup tags found in the given `text` by the given token.
     By default `token` is an empty string so it just removes all tags.
 
@@ -111,10 +115,10 @@ def replace_tags(text, token='', encoding=None):
     Examples:
 
     >>> import w3lib.html
-    >>> w3lib.html.replace_tags(u'This text contains <a>some tag</a>')
-    u'This text contains some tag'
+    >>> w3lib.html.replace_tags('This text contains <a>some tag</a>')
+    'This text contains some tag'
     >>> w3lib.html.replace_tags('<p>Je ne parle pas <b>fran\\xe7ais</b></p>', ' -- ', 'latin-1')
-    u' -- Je ne parle pas  -- fran\\xe7ais --  -- '
+    ' -- Je ne parle pas  -- fran\\xe7ais --  -- '
     >>>
 
     """
@@ -122,22 +126,30 @@ def replace_tags(text, token='', encoding=None):
     return _tag_re.sub(token, to_unicode(text, encoding))
 
 
-_REMOVECOMMENTS_RE = re.compile(u'<!--.*?(?:-->|$)', re.DOTALL)
-def remove_comments(text, encoding=None):
-    """ Remove HTML Comments.
+_REMOVECOMMENTS_RE = re.compile("<!--.*?(?:-->|$)", re.DOTALL)
+
+
+def remove_comments(text: AnyStr, encoding: Optional[str] = None) -> str:
+    """Remove HTML Comments.
 
     >>> import w3lib.html
     >>> w3lib.html.remove_comments(b"test <!--textcoment--> whatever")
-    u'test  whatever'
+    'test  whatever'
     >>>
 
     """
 
-    text = to_unicode(text, encoding)
-    return _REMOVECOMMENTS_RE.sub(u'', text)
+    utext = to_unicode(text, encoding)
+    return _REMOVECOMMENTS_RE.sub("", utext)
 
-def remove_tags(text, which_ones=(), keep=(), encoding=None):
-    """ Remove HTML Tags only.
+
+def remove_tags(
+    text: AnyStr,
+    which_ones: Iterable[str] = (),
+    keep: Iterable[str] = (),
+    encoding: Optional[str] = None,
+) -> str:
+    """Remove HTML Tags only.
 
     `which_ones` and `keep` are both tuples, there are four cases:
 
@@ -156,55 +168,56 @@ def remove_tags(text, which_ones=(), keep=(), encoding=None):
     >>> import w3lib.html
     >>> doc = '<div><p><b>This is a link:</b> <a href="http://www.example.com">example</a></p></div>'
     >>> w3lib.html.remove_tags(doc)
-    u'This is a link: example'
+    'This is a link: example'
     >>>
 
     Keep only some tags:
 
     >>> w3lib.html.remove_tags(doc, keep=('div',))
-    u'<div>This is a link: example</div>'
+    '<div>This is a link: example</div>'
     >>>
 
     Remove only specific tags:
 
     >>> w3lib.html.remove_tags(doc, which_ones=('a','b'))
-    u'<div><p>This is a link: example</p></div>'
+    '<div><p>This is a link: example</p></div>'
     >>>
 
     You can't remove some and keep some:
 
     >>> w3lib.html.remove_tags(doc, which_ones=('a',), keep=('p',))
     Traceback (most recent call last):
-      File "<stdin>", line 1, in <module>
-      File "/usr/local/lib/python2.7/dist-packages/w3lib/html.py", line 101, in remove_tags
-        assert not (which_ones and keep), 'which_ones and keep can not be given at the same time'
-    AssertionError: which_ones and keep can not be given at the same time
+        ...
+    ValueError: Cannot use both which_ones and keep
     >>>
 
     """
-
-    assert not (which_ones and keep), 'which_ones and keep can not be given at the same time'
+    if which_ones and keep:
+        raise ValueError("Cannot use both which_ones and keep")
 
     which_ones = {tag.lower() for tag in which_ones}
     keep = {tag.lower() for tag in keep}
 
-    def will_remove(tag):
+    def will_remove(tag: str) -> bool:
         tag = tag.lower()
         if which_ones:
             return tag in which_ones
         else:
             return tag not in keep
 
-    def remove_tag(m):
+    def remove_tag(m: Match) -> str:
         tag = m.group(1)
-        return u'' if will_remove(tag) else m.group(0)
+        return "" if will_remove(tag) else m.group(0)
 
-    regex = '</?([^ >/]+).*?>'
+    regex = "</?([^ >/]+).*?>"
     retags = re.compile(regex, re.DOTALL | re.IGNORECASE)
 
     return retags.sub(remove_tag, to_unicode(text, encoding))
 
-def remove_tags_with_content(text, which_ones=(), encoding=None):
+
+def remove_tags_with_content(
+    text: AnyStr, which_ones: Iterable[str] = (), encoding: Optional[str] = None
+) -> str:
     """Remove tags and their content.
 
     `which_ones` is a tuple of which tags to remove including their content.
@@ -213,21 +226,25 @@ def remove_tags_with_content(text, which_ones=(), encoding=None):
     >>> import w3lib.html
     >>> doc = '<div><p><b>This is a link:</b> <a href="http://www.example.com">example</a></p></div>'
     >>> w3lib.html.remove_tags_with_content(doc, which_ones=('b',))
-    u'<div><p> <a href="http://www.example.com">example</a></p></div>'
+    '<div><p> <a href="http://www.example.com">example</a></p></div>'
     >>>
 
     """
 
-    text = to_unicode(text, encoding)
+    utext = to_unicode(text, encoding)
     if which_ones:
-        tags = '|'.join([r'<%s\b.*?</%s>|<%s\s*/>' % (tag, tag, tag) for tag in which_ones])
+        tags = "|".join([rf"<{tag}\b.*?</{tag}>|<{tag}\s*/>" for tag in which_ones])
         retags = re.compile(tags, re.DOTALL | re.IGNORECASE)
-        text = retags.sub(u'', text)
-    return text
+        utext = retags.sub("", utext)
+    return utext
 
 
-def replace_escape_chars(text, which_ones=('\n', '\t', '\r'), replace_by=u'', \
-        encoding=None):
+def replace_escape_chars(
+    text: AnyStr,
+    which_ones: Iterable[str] = ("\n", "\t", "\r"),
+    replace_by: StrOrBytes = "",
+    encoding: Optional[str] = None,
+) -> str:
     """Remove escape characters.
 
     `which_ones` is a tuple of which escape characters we want to remove.
@@ -238,12 +255,18 @@ def replace_escape_chars(text, which_ones=('\n', '\t', '\r'), replace_by=u'', \
 
     """
 
-    text = to_unicode(text, encoding)
+    utext = to_unicode(text, encoding)
     for ec in which_ones:
-        text = text.replace(ec, to_unicode(replace_by, encoding))
-    return text
+        utext = utext.replace(ec, to_unicode(replace_by, encoding))
+    return utext
 
-def unquote_markup(text, keep=(), remove_illegal=True, encoding=None):
+
+def unquote_markup(
+    text: AnyStr,
+    keep: Iterable[str] = (),
+    remove_illegal: bool = True,
+    encoding: Optional[str] = None,
+) -> str:
     """
     This function receives markup as a text (always a unicode string or
     a UTF-8 encoded string) and does the following:
@@ -255,7 +278,7 @@ def unquote_markup(text, keep=(), remove_illegal=True, encoding=None):
 
     """
 
-    def _get_fragments(txt, pattern):
+    def _get_fragments(txt: str, pattern: Pattern) -> Iterable[Union[str, Match]]:
         offset = 0
         for match in pattern.finditer(txt):
             match_s, match_e = match.span(1)
@@ -264,18 +287,23 @@ def unquote_markup(text, keep=(), remove_illegal=True, encoding=None):
             offset = match_e
         yield txt[offset:]
 
-    text = to_unicode(text, encoding)
-    ret_text = u''
-    for fragment in _get_fragments(text, _cdata_re):
-        if isinstance(fragment, six.string_types):
+    utext = to_unicode(text, encoding)
+    ret_text = ""
+    for fragment in _get_fragments(utext, _cdata_re):
+        if isinstance(fragment, str):
             # it's not a CDATA (so we try to remove its entities)
-            ret_text += replace_entities(fragment, keep=keep, remove_illegal=remove_illegal)
+            ret_text += replace_entities(
+                fragment, keep=keep, remove_illegal=remove_illegal
+            )
         else:
             # it's a CDATA (so we just extract its content)
-            ret_text += fragment.group('cdata_d')
+            ret_text += fragment.group("cdata_d")
     return ret_text
 
-def get_base_url(text, baseurl='', encoding='utf-8'):
+
+def get_base_url(
+    text: AnyStr, baseurl: StrOrBytes = "", encoding: str = "utf-8"
+) -> str:
     """Return the base url if declared in the given HTML `text`,
     relative to the given base url.
 
@@ -283,17 +311,22 @@ def get_base_url(text, baseurl='', encoding='utf-8'):
 
     """
 
-    text = to_unicode(text, encoding)
-    m = _baseurl_re.search(text)
+    utext = to_unicode(text, encoding)
+    m = _baseurl_re.search(utext)
     if m:
-        return moves.urllib.parse.urljoin(
-            safe_url_string(baseurl),
-            safe_url_string(m.group(1), encoding=encoding)
+        return urljoin(
+            safe_url_string(baseurl), safe_url_string(m.group(1), encoding=encoding)
         )
     else:
         return safe_url_string(baseurl)
 
-def get_meta_refresh(text, baseurl='', encoding='utf-8', ignore_tags=('script', 'noscript')):
+
+def get_meta_refresh(
+    text: AnyStr,
+    baseurl: str = "",
+    encoding: str = "utf-8",
+    ignore_tags: Iterable[str] = ("script", "noscript"),
+) -> Tuple[Optional[float], Optional[str]]:
     """Return  the http-equiv parameter of the HTML meta element from the given
     HTML text and return a tuple ``(interval, url)`` where interval is an integer
     containing the delay in seconds (or zero if not present) and url is a
@@ -303,26 +336,24 @@ def get_meta_refresh(text, baseurl='', encoding='utf-8', ignore_tags=('script',
 
     """
 
-    if six.PY2:
-        baseurl = to_bytes(baseurl, encoding)
     try:
-        text = to_unicode(text, encoding)
+        utext = to_unicode(text, encoding)
     except UnicodeDecodeError:
         print(text)
         raise
-    text = remove_tags_with_content(text, ignore_tags)
-    text = remove_comments(replace_entities(text))
-    m = _meta_refresh_re.search(text)
+    utext = remove_tags_with_content(utext, ignore_tags)
+    utext = remove_comments(replace_entities(utext))
+    m = _meta_refresh_re.search(utext) or _meta_refresh_re2.search(utext)
     if m:
-        interval = float(m.group('int'))
-        url = safe_url_string(m.group('url').strip(' "\''), encoding)
-        url = moves.urllib.parse.urljoin(baseurl, url)
+        interval = float(m.group("int"))
+        url = safe_url_string(m.group("url").strip(" \"'"), encoding)
+        url = urljoin(baseurl, url)
         return interval, url
     else:
         return None, None
 
 
-def strip_html5_whitespace(text):
+def strip_html5_whitespace(text: str) -> str:
     r"""
     Strip all leading and trailing space characters (as defined in
     https://www.w3.org/TR/html5/infrastructure.html#space-character).
